@@ -41,7 +41,8 @@ from pysollib.stack import \
     WasteTalonStack, \
     Yukon_AC_RowStack, \
     Yukon_BO_RowStack, \
-    Yukon_SS_RowStack
+    Yukon_SS_RowStack, \
+    isAlternateColorSequence
 from pysollib.util import ANY_SUIT, KING
 
 
@@ -56,18 +57,25 @@ class Yukon(Game):
     RowStack_Class = StackWrapper(Yukon_AC_RowStack, base_rank=KING)
     Hint_Class = Yukon_Hint
 
-    def createGame(self, **layout):
+    DIFF_SUITS = True
+
+    def createGame(self, rows=7, **layout):
         # create layout
         l, s = Layout(self), self.s
-        kwdefault(layout, rows=7, texts=0, playcards=25)
+        kwdefault(layout, rows=rows, texts=0, playcards=25)
         self.Layout_Method(l, **layout)
         self.setSize(l.size[0], l.size[1])
         # create stacks
         s.talon = self.Talon_Class(l.s.talon.x, l.s.talon.y, self)
         for r in l.s.foundations:
-            s.foundations.append(
-                self.Foundation_Class(
-                    r.x, r.y, self, suit=r.suit, max_move=0))
+            if self.DIFF_SUITS:
+                s.foundations.append(
+                    self.Foundation_Class(
+                        r.x, r.y, self, suit=r.suit, max_move=0))
+            else:
+                s.foundations.append(
+                    self.Foundation_Class(
+                        r.x, r.y, self, suit=ANY_SUIT, max_move=0))
         for r in l.s.rows:
             s.rows.append(self.RowStack_Class(r.x, r.y, self))
         # default
@@ -370,6 +378,23 @@ class TripleRussianSolitaire(TripleYukon):
     shallHighlightMatch = Game._shallHighlightMatch_SS
 
 
+# ************************************************************************
+# * Quadruple Yukon
+# ************************************************************************
+
+class QuadrupleYukon(Yukon):
+    def createGame(self):
+        Yukon.createGame(self, rows=16, playcards=34)
+
+    def startGame(self):
+        for i in range(1, len(self.s.rows)):
+            self.s.talon.dealRow(rows=self.s.rows[i:], flip=0, frames=0)
+        self.s.talon.dealRow(rows=self.s.rows[1:13], flip=1, frames=0)
+        for i in range(4):
+            self.s.talon.dealRow(rows=self.s.rows[1:], flip=1, frames=0)
+        self._startAndDealRow()
+
+
 # ************************************************************************
 # * Ten Across
 # ************************************************************************
@@ -448,6 +473,7 @@ class Panopticon(TenAcross):
 # ************************************************************************
 # * Australian Patience
 # * Tasmanian Patience
+# * Canberra
 # * Raw Prawn
 # * Bim Bom
 # ************************************************************************
@@ -479,6 +505,11 @@ class TasmanianPatience(AustralianPatience):
         AustralianPatience.createGame(self, max_rounds=-1, num_deal=3)
 
 
+class Canberra(AustralianPatience):
+    def createGame(self):
+        AustralianPatience.createGame(self, max_rounds=2)
+
+
 class RawPrawn(AustralianPatience):
     RowStack_Class = Yukon_SS_RowStack
 
@@ -539,6 +570,8 @@ class Queensland(Yukon):
 # ************************************************************************
 # * Russian Spider
 # * Double Russian Spider
+# * Kiev
+# * Dnieper
 # ************************************************************************
 
 class RussianSpider_RowStack(Yukon_SS_RowStack):  # Spider_SS_RowStack
@@ -556,31 +589,35 @@ class RussianSpider(RussianSolitaire):
     RowStack_Class = StackWrapper(RussianSpider_RowStack, base_rank=KING)
     Foundation_Class = Spider_SS_Foundation
 
-    def createGame(self, rows=7):
-        # create layout
-        l, s = Layout(self), self.s
-        l.yukonLayout(rows=rows, texts=0, playcards=25)
-        self.setSize(l.size[0], l.size[1])
-        # create stacks
-        s.talon = self.Talon_Class(l.s.talon.x, l.s.talon.y, self)
-        for r in l.s.foundations:
-            s.foundations.append(
-                self.Foundation_Class(
-                    r.x, r.y, self, suit=ANY_SUIT, max_move=0))
-        for r in l.s.rows:
-            s.rows.append(self.RowStack_Class(r.x, r.y, self))
-        # default
-        l.defaultAll()
+    DIFF_SUITS = False
 
 
 class DoubleRussianSpider(RussianSpider, DoubleRussianSolitaire):
     def createGame(self):
-        RussianSpider.createGame(self, rows=10)
+        Yukon.createGame(self, rows=10)
 
     def startGame(self):
         DoubleRussianSolitaire.startGame(self)
 
 
+class Kiev(RussianSpider):
+    RowStack_Class = RussianSpider_RowStack
+    Layout_Method = staticmethod(Layout.klondikeLayout)
+    Talon_Class = DealRowTalonStack
+
+    def createGame(self):
+        return Yukon.createGame(self, waste=0, texts=1)
+
+    def startGame(self):
+        for i in range(3):
+            self.s.talon.dealRow(flip=0, frames=0)
+        self._startAndDealRow()
+
+
+class Dnieper(Kiev):
+    RowStack_Class = StackWrapper(RussianSpider_RowStack, mod=13)
+
+
 # ************************************************************************
 # * Brisbane
 # ************************************************************************
@@ -687,7 +724,49 @@ class Wave(Game):
     shallHighlightMatch = Game._shallHighlightMatch_AC
 
 
-class YukonicPlague(Yukon):
+# ************************************************************************
+# * Yukon Kings
+# ************************************************************************
+
+class YukonKings(Yukon):
+
+    def createGame(self, playcards=25):
+        l, s = Layout(self), self.s
+        self.setSize(l.XM + (7 * l.XS),
+                     l.YM + l.YS + playcards * l.YOFFSET)
+
+        x, y = l.XM, l.YM
+        for i in range(7):
+            s.rows.append(self.RowStack_Class(x, y, self))
+            x += l.XS
+
+        x, y = l.XM, self.height - l.YS
+        s.talon = InitialDealTalonStack(x, y, self)
+
+        l.defaultStackGroups()
+
+    def isGameWon(self):
+        cardsPlayed = False
+        for s in self.s.rows:
+            if s.cards:
+                if len(s.cards) != 13 or not isAlternateColorSequence(s.cards):
+                    return False
+                cardsPlayed = True
+        if not cardsPlayed:
+            return False
+        return True
+
+
+# ************************************************************************
+# * Yukon Cells
+# * Yukonic Plague
+# ************************************************************************
+
+class YukonCells(Yukon):
+    Reserve_Stack = ReserveStack
+
+    RESERVES = 2
+    RESERVE_TEXT = False
 
     def createGame(self):
         # create layout
@@ -706,10 +785,14 @@ class YukonicPlague(Yukon):
         if w2 + 13 * l.XOFFSET > w1:
             l.XOFFSET = int((w1 - w2) / 13)
 
-        reserve = OpenStack(x * 3, y, self)
-        reserve.CARD_XOFFSET = l.XOFFSET
-        l.createText(reserve, "sw")
-        s.reserves.append(reserve)
+        x2 = x * 3
+        for i in range(self.RESERVES):
+            reserve = self.Reserve_Stack(x2, y, self)
+            reserve.CARD_XOFFSET = l.XOFFSET
+            if self.RESERVE_TEXT:
+                l.createText(reserve, "sw")
+            s.reserves.append(reserve)
+            x2 += l.XS
 
         y += l.YS
         for i in range(ROWS):
@@ -719,7 +802,8 @@ class YukonicPlague(Yukon):
         # Don't know why this is necessary for the Yukon layout.
         # But we should probably figure out how to get this to work
         # like other games.
-        self.setRegion(self.s.rows, (-999, -999, x - l.CW // 2, 999999))
+        self.setRegion(self.s.rows + self.s.reserves,
+                       (-999, -999, x - l.CW // 2, 999999))
 
         # create foundations
         y = l.YM
@@ -735,6 +819,13 @@ class YukonicPlague(Yukon):
         self.setSize(l.XM + 8 * l.XS, h)
         l.defaultAll()
 
+
+class YukonicPlague(YukonCells):
+    Reserve_Stack = OpenStack
+
+    RESERVES = 1
+    RESERVE_TEXT = True
+
     def startGame(self):
         for i in range(13):
             self.s.talon.dealRow(rows=self.s.reserves, frames=0)
@@ -779,7 +870,8 @@ registerGame(GameInfo(285, Panopticon, "Panopticon",
 registerGame(GameInfo(339, Moosehide, "Moosehide",
                       GI.GT_YUKON, 1, 0, GI.SL_MOSTLY_SKILL))
 registerGame(GameInfo(387, Roslin, "Roslin",
-                      GI.GT_YUKON, 1, 0, GI.SL_MOSTLY_SKILL))
+                      GI.GT_YUKON, 1, 0, GI.SL_MOSTLY_SKILL,
+                      altnames=("Roslyn",)))
 registerGame(GameInfo(447, AustralianPatience, "Australian Patience",
                       GI.GT_YUKON, 1, 0, GI.SL_BALANCED,
                       altnames=('Outback Patience',)))
@@ -810,3 +902,17 @@ registerGame(GameInfo(826, YukonicPlague, "Yukonic Plague",
                       GI.GT_YUKON, 1, 0, GI.SL_BALANCED))
 registerGame(GameInfo(857, TasmanianPatience, "Tasmanian Patience",
                       GI.GT_YUKON, 1, -1, GI.SL_BALANCED))
+registerGame(GameInfo(897, Kiev, "Kiev",
+                      GI.GT_SPIDER, 1, 0, GI.SL_BALANCED,
+                      altnames=('Kyiv',)))
+registerGame(GameInfo(914, Canberra, "Canberra",
+                      GI.GT_YUKON, 1, 1, GI.SL_BALANCED))
+registerGame(GameInfo(919, Dnieper, "Dnieper",
+                      GI.GT_SPIDER, 1, 0, GI.SL_BALANCED,
+                      altnames=('Dnipro',)))
+registerGame(GameInfo(925, YukonCells, "Yukon Cells",
+                      GI.GT_YUKON, 1, 0, GI.SL_BALANCED))
+registerGame(GameInfo(936, YukonKings, "Yukon Kings",
+                      GI.GT_YUKON, 1, 0, GI.SL_BALANCED))
+registerGame(GameInfo(942, QuadrupleYukon, "Quadruple Yukon",
+                      GI.GT_YUKON, 4, 0, GI.SL_BALANCED))
