@@ -86,6 +86,73 @@ def createStatusbarMenu(menubar, menu):
             command=lambda m=menubar, w=comp: m.mOptStatusbarConfig(w))
 
 
+def createOtherGraphicsMenu(menubar, menu):
+    tearoff = menu.cget('tearoff')
+    data_dir = os.path.join(menubar.app.dataloader.dir, 'images', 'demo')
+    submenu = MfxMenu(menu, label=n_('&Demo logo'), tearoff=tearoff)
+    styledirs = os.listdir(data_dir)
+    styledirs.append("none")
+    styledirs.sort()
+    for f in styledirs:
+        d = os.path.join(data_dir, f)
+        if (os.path.isdir(d) and os.path.exists(os.path.join(d))) \
+                or f == "none":
+            name = f.replace('_', ' ').capitalize()
+            submenu.add_radiobutton(
+                label=name,
+                variable=menubar.tkopt.demo_logo_style,
+                value=f, command=menubar.mOptDemoLogoStyle)
+    data_dir = os.path.join(menubar.app.dataloader.dir, 'images', 'dialog')
+    submenu = MfxMenu(menu, label=n_('D&ialog icons'), tearoff=tearoff)
+    styledirs = os.listdir(data_dir)
+    styledirs.sort()
+    for f in styledirs:
+        d = os.path.join(data_dir, f)
+        if os.path.isdir(d) and os.path.exists(os.path.join(d)):
+            name = f.replace('_', ' ').capitalize()
+            submenu.add_radiobutton(
+                label=name,
+                variable=menubar.tkopt.dialog_icon_style,
+                value=f, command=menubar.mOptDialogIconStyle)
+    data_dir = os.path.join(menubar.app.dataloader.dir, 'images', 'pause')
+    submenu = MfxMenu(menu, label=n_('&Pause text'), tearoff=tearoff)
+    styledirs = os.listdir(data_dir)
+    styledirs.sort()
+    for f in styledirs:
+        d = os.path.join(data_dir, f)
+        if os.path.isdir(d) and os.path.exists(os.path.join(d)):
+            name = f.replace('_', ' ').capitalize()
+            submenu.add_radiobutton(
+                label=name,
+                variable=menubar.tkopt.pause_text_style,
+                value=f, command=menubar.mOptPauseTextStyle)
+    data_dir = os.path.join(menubar.app.dataloader.dir, 'images',
+                            'redealicons')
+    submenu = MfxMenu(menu, label=n_('&Redeal icons'), tearoff=tearoff)
+    styledirs = os.listdir(data_dir)
+    styledirs.sort()
+    for f in styledirs:
+        d = os.path.join(data_dir, f)
+        if os.path.isdir(d) and os.path.exists(os.path.join(d)):
+            name = f.replace('_', ' ').capitalize()
+            submenu.add_radiobutton(
+                label=name,
+                variable=menubar.tkopt.redeal_icon_style,
+                value=f, command=menubar.mOptRedealIconStyle)
+    data_dir = os.path.join(menubar.app.dataloader.dir, 'images', 'tree')
+    submenu = MfxMenu(menu, label=n_('&Tree icons'), tearoff=tearoff)
+    styledirs = os.listdir(data_dir)
+    styledirs.sort()
+    for f in styledirs:
+        d = os.path.join(data_dir, f)
+        if os.path.isdir(d) and os.path.exists(os.path.join(d)):
+            name = f.replace('_', ' ').capitalize()
+            submenu.add_radiobutton(
+                label=name,
+                variable=menubar.tkopt.tree_icon_style,
+                value=f, command=menubar.mOptTreeIconStyle)
+
+
 def createResamplingMenu(menubar, menu):
     tearoff = menu.cget('tearoff')
     submenu = MfxMenu(menu, label=n_('R&esampling'), tearoff=tearoff)
@@ -260,6 +327,11 @@ class PysolMenubarTkCommon:
             helpbar=tkinter.BooleanVar(),
             splashscreen=tkinter.BooleanVar(),
             demo_logo=tkinter.BooleanVar(),
+            demo_logo_style=tkinter.StringVar(),
+            pause_text_style=tkinter.StringVar(),
+            redeal_icon_style=tkinter.StringVar(),
+            dialog_icon_style=tkinter.StringVar(),
+            tree_icon_style=tkinter.StringVar(),
             mouse_type=tkinter.StringVar(),
             mouse_undo=tkinter.BooleanVar(),
             negative_bottom=tkinter.BooleanVar(),
@@ -324,6 +396,14 @@ class PysolMenubarTkCommon:
         # tkopt.num_cards.set(opt.num_cards)
         # tkopt.helpbar.set(opt.helpbar)
         tkopt.demo_logo.set(opt.demo_logo)
+        if opt.demo_logo:
+            tkopt.demo_logo_style.set(opt.demo_logo_style)
+        else:
+            tkopt.demo_logo_style.set("none")
+        tkopt.pause_text_style.set(opt.pause_text_style)
+        tkopt.redeal_icon_style.set(opt.redeal_icon_style)
+        tkopt.dialog_icon_style.set(opt.dialog_icon_style)
+        tkopt.tree_icon_style.set(opt.tree_icon_style)
         tkopt.splashscreen.set(opt.splashscreen)
         tkopt.mouse_type.set(opt.mouse_type)
         tkopt.mouse_undo.set(opt.mouse_undo)
@@ -347,6 +427,10 @@ class PysolMenubarTkCommon:
             self._connect_game_find_card_dialog(game)
         else:
             self._destroy_find_card_dialog()
+        if game.canShowFullPicture():
+            self._connect_game_full_picture_dialog(game)
+        else:
+            self._destroy_full_picture_dialog()
         self._connect_game_solver_dialog(game)
 
     # create a GTK-like path
@@ -534,6 +618,9 @@ class PysolMenubarTkCommon:
         menu.add_command(
             label=n_("&Find card..."),
             command=self.mFindCard, accelerator="F3")
+        menu.add_command(
+            label=n_("Sh&ow full picture..."),
+            command=self.mFullPicture)
         menu.add_separator()
         menu.add_command(
             label=n_("&Demo"),
@@ -763,18 +850,19 @@ class PysolMenubarTkCommon:
         createToolbarMenu(self, submenu)
         submenu = MfxMenu(menu, label=n_("Stat&usbar"))
         createStatusbarMenu(self, submenu)
+        submenu = MfxMenu(menu, label=n_("Othe&r graphics"))
+        createOtherGraphicsMenu(self, submenu)
         if not USE_PIL:
+            menu.add_separator()
             menu.add_checkbutton(
                 label=n_("Save games &geometry"),
                 variable=self.tkopt.save_games_geometry,
                 command=self.mOptSaveGamesGeometry)
-        menu.add_checkbutton(
-            label=n_("&Demo logo"), variable=self.tkopt.demo_logo,
-            command=self.mOptDemoLogo)
-        menu.add_checkbutton(
-            label=n_("Startup splash sc&reen"),
-            variable=self.tkopt.splashscreen,
-            command=self.mOptSplashscreen)
+
+        # menu.add_checkbutton(
+        #     label=n_("Startup splash sc&reen"),
+        #     variable=self.tkopt.splashscreen,
+        #     command=self.mOptSplashscreen)
         #  menu.add_separator()
         #  menu.add_command(label="Save options", command=self.mOptSave)
 
@@ -960,7 +1048,7 @@ class PysolMenubarTkCommon:
                                       self.tkopt.gameid)
 
     def _addSelectGameSubMenu(self, games, menu, select_data,
-                              command, variable):
+                              command, variable, short_name=False):
         # print select_data
         need_sep = 0
         for label, select_func in select_data:
@@ -974,7 +1062,8 @@ class PysolMenubarTkCommon:
                 menu.add_separator()
                 need_sep = 0
             submenu = MfxMenu(menu, label=label)
-            self._addSelectGameSubSubMenu(g, submenu, command, variable)
+            self._addSelectGameSubSubMenu(g, submenu, command, variable,
+                                          short_name=short_name)
 
     def _getNumGames(self, games, select_data):
         ngames = 0
@@ -996,38 +1085,24 @@ class PysolMenubarTkCommon:
         mahjongg_games.sort(key=sort_func)
         #
         menu = MfxMenu(menu, label=n_("&Mahjongg games"))
-
-        def add_menu(games, c0, c1, menu=menu,
-                     variable=variable, command=command):
-            if not games:
-                return
-            label = c0 + ' - ' + c1
-            if c0 == c1:
-                label = c0
+        n, d = 0, self.cb_max
+        i = 0
+        while True:
+            if self.progress:
+                self.progress.update(step=1)
+            columnbreak = i > 0 and (i % d) == 0
+            i += 1
+            if not mahjongg_games[n:n + d]:
+                break
+            m = min(n + d - 1, len(mahjongg_games) - 1)
+            label = mahjongg_games[n].short_name[:3] + ' - ' + \
+                mahjongg_games[m].short_name[:3]
             submenu = MfxMenu(menu, label=label, name=None)
-            self._addSelectGameSubSubMenu(games, submenu, command,
-                                          variable, short_name=True)
-
-        games = {}
-        for gi in mahjongg_games:
-            c = gi.short_name.strip()[0]
-            if c in games:
-                games[c].append(gi)
-            else:
-                games[c] = [gi]
-        games = list(games.items())
-        games.sort()
-        g0 = []
-        c0 = c1 = games[0][0]
-        for c, g1 in games:
-            if len(g0)+len(g1) >= self.cb_max:
-                add_menu(g0, c0, c1)
-                g0 = g1
-                c0 = c1 = c
-            else:
-                g0 += g1
-                c1 = c
-        add_menu(g0, c0, c1)
+            self._addSelectGameSubSubMenu(mahjongg_games[n:n + d], submenu,
+                                          command, variable, short_name=True)
+            n += d
+            if columnbreak:
+                menu.entryconfigure(i, columnbreak=columnbreak)
 
     def _addSelectPopularGameSubMenu(self, games, menu, command, variable):
         def select_func(gi):
@@ -1132,9 +1207,11 @@ class PysolMenubarTkCommon:
 
     def mSelectGame(self, *args):
         self._mSelectGame(self.tkopt.gameid.get())
+        self.tkopt.gameid.set(self.game.id)
 
     def mSelectGamePopular(self, *args):
         self._mSelectGame(self.tkopt.gameid_popular.get())
+        self.tkopt.gameid_popular.set(self.game.id)
 
     def _mSelectGameDialog(self, d):
         if d.status == 0 and d.button == 0 and d.gameid != self.game.id:
@@ -1185,14 +1262,23 @@ class PysolMenubarTkCommon:
 
     def updateFavoriteGamesMenu(self):
         gameids = self.app.opt.favorite_gameid
-        submenu = self.menupath[".menubar.file.favoritegames"][2]
+        menu, index, submenu = self.menupath[".menubar.file.favoritegames"]
         games = []
         for id in gameids:
             gi = self.app.getGameInfo(id)
             if gi:
                 games.append(gi)
-        self.updateGamesMenu(submenu, games)
+
+        def sort_func(gi):
+            return gi.name
+
+        games.sort(key=sort_func)
         state = self._getEnabledState
+        if len(games) > 0:
+            self.updateGamesMenu(submenu, games)
+            menu.entryconfig(index, state=state(True))
+        else:
+            menu.entryconfig(index, state=state(False))
         in_favor = self.app.game.id in gameids
         menu, index, submenu = self.menupath[".menubar.file.addtofavorites"]
         menu.entryconfig(index, state=state(not in_favor))
@@ -1208,6 +1294,9 @@ class PysolMenubarTkCommon:
             if gi:
                 games.append(gi)
         self.updateGamesMenu(submenu, games)
+        submenu.add_separator()
+        submenu.add_command(label=n_("&Clear recent games"),
+                            command=self.mClearRecent)
 
     def updateCustomGamesMenu(self):
         menu = self.menupath[".menubar.select.customgames"][2]
@@ -1312,6 +1401,17 @@ class PysolMenubarTkCommon:
             self.app.opt.favorite_gameid.remove(gameid)
             self.updateFavoriteGamesMenu()
 
+    def mClearRecent(self, *event):
+        if self._cancelDrag(break_pause=False):
+            return
+        if not self.game.areYouSure(_("Clear recent games"),
+                                    _("Clear the recent games list?")):
+            return
+        gameid = self.app.game.id
+        self.app.opt.recent_gameid = []
+        self.app.opt.recent_gameid.append(gameid)
+        self.updateRecentGamesMenu(self.app.opt.recent_gameid)
+
     def mOpen(self, *event):
         if self._cancelDrag(break_pause=False):
             return
@@ -1357,7 +1457,7 @@ Unsupported game for export.
             filename += ".board"
         idir, ifile = os.path.split(os.path.normpath(filename))
         if not idir:
-            idir = self.app.dn.savegames
+            idir = self.app.dn.boards
         # print self.game.filename, ifile
         d = tkinter_tkfiledialog.SaveAs()
         filename = d.show(filetypes=self.FILETYPES,
@@ -1390,7 +1490,7 @@ Unsupported game for import.
         else:
             idir, ifile = "", ""
         if not idir:
-            idir = self.app.dn.savegames
+            idir = self.app.dn.boards
         d = tkinter_tkfiledialog.Open()
         key = 'PYSOL_DEBUG_IMPORT'
         if key not in os.environ:
@@ -1846,6 +1946,21 @@ Unsupported game for import.
     def mOptStatusbarConfig(self, w):
         self.statusbarConfig(w, self.tkopt.statusbar_vars[w].get())
 
+    def mOptDemoLogoStyle(self, *event):
+        self.setDemoLogoStyle(self.tkopt.demo_logo_style.get())
+
+    def mOptDialogIconStyle(self, *event):
+        self.setDialogIconStyle(self.tkopt.dialog_icon_style.get())
+
+    def mOptPauseTextStyle(self, *event):
+        self.setPauseTextStyle(self.tkopt.pause_text_style.get())
+
+    def mOptRedealIconStyle(self, *event):
+        self.setRedealIconStyle(self.tkopt.redeal_icon_style.get())
+
+    def mOptTreeIconStyle(self, *event):
+        self.setTreeIconStyle(self.tkopt.tree_icon_style.get())
+
     def mOptNumCards(self, *event):
         if self._cancelDrag(break_pause=False):
             return
@@ -1933,6 +2048,62 @@ Unsupported game for import.
             self.game.updateStatus(player=self.app.opt.player)
             self.top.update_idletasks()
 
+    def setDemoLogoStyle(self, style):
+        if self._cancelDrag(break_pause=False):
+            return
+        if style == "none":
+            self.app.opt.demo_logo = False
+        else:
+            self.app.opt.demo_logo = True
+            self.app.opt.demo_logo_style = style
+            self.tkopt.demo_logo_style.set(style)         # update radiobutton
+            self.app.loadImages2()
+            self.app.loadImages4()
+            self.app.updateCardset()
+            self.game.endGame(bookmark=1)
+            self.game.quitGame(bookmark=1)
+
+    def setDialogIconStyle(self, style):
+        if self._cancelDrag(break_pause=False):
+            return
+        self.app.opt.dialog_icon_style = style
+        self.tkopt.dialog_icon_style.set(style)           # update radiobutton
+        self.app.loadImages1()
+        self.app.loadImages4()
+
+    def setPauseTextStyle(self, style):
+        if self._cancelDrag(break_pause=False):
+            return
+        self.app.opt.pause_text_style = style
+        self.tkopt.pause_text_style.set(style)            # update radiobutton
+        self.app.loadImages2()
+        self.app.loadImages4()
+        self.app.updateCardset()
+        self.game.endGame(bookmark=1)
+        self.game.quitGame(bookmark=1)
+
+    def setRedealIconStyle(self, style):
+        if self._cancelDrag(break_pause=False):
+            return
+        self.app.opt.redeal_icon_style = style
+        self.tkopt.redeal_icon_style.set(style)           # update radiobutton
+        self.app.loadImages2()
+        self.app.loadImages4()
+        self.app.updateCardset()
+        self.game.endGame(bookmark=1)
+        self.game.quitGame(bookmark=1)
+
+    def setTreeIconStyle(self, style):
+        if self._cancelDrag(break_pause=False):
+            return
+        self.app.opt.tree_icon_style = style
+        self.tkopt.tree_icon_style.set(style)           # update radiobutton
+        self.app.loadImages3()
+        self.app.loadImages4()
+        self.app.updateCardset()
+        self.game.endGame(bookmark=1)
+        self.game.quitGame(bookmark=1)
+
     def wizardDialog(self, edit=False):
         from pysollib.wizardutil import write_game, reset_wizard
         WizardDialog = self._calcWizardDialog()
